import React, { Component } from "react";
import Fade from "@material-ui/core/Fade";
import Slide from "@material-ui/core/Slide";
import IconButton from "@material-ui/core/IconButton";
import { withStyles } from "@material-ui/core/styles";
import autoBind from "auto-bind";
import FiberManualRecordIcon from "@material-ui/icons/FiberManualRecord";
import NavigateBeforeIcon from "@material-ui/icons/NavigateBefore";
import NavigateNextIcon from "@material-ui/icons/NavigateNext";
import { useSwipeable } from "react-swipeable";
import Button from "@material-ui/core/Button";
import FastForwardIcon from "@material-ui/icons/FastForward";
import FastRewindIcon from "@material-ui/icons/FastRewind";
import PlayArrowIcon from "@material-ui/icons/PlayArrow";

const styles = {
  root: {
    // position: "relative",
    overflow: "hidden",
    backgroundColor: "#e0e0e0",
    padding: "1rem",
  },
  container: {
    position: "relative",
  },
  indicators: {
    width: "100%",
    marginTop: "10px",
    textAlign: "center",
  },
  indicator: {
    fontSize: "15px",
    cursor: "pointer",
    transition: "200ms",
    color: "#afafaf",
    "&:hover": {
      color: "#1f1f1f",
    },
    "&:active": {
      color: "#1f1f1f",
    },
  },
  active: {
    color: "#494949",
  },
  buttonWrapper: {
    position: "absolute",
    height: "100px",
    margin: 0,
    backgroundColor: "transparent",
    top: "0px",
  },
  fullHeightHoverWrapper: {
    height: "calc(10% - 20px - 10px) !important",
    top: "-10% !important",
  },
  button: {
    margin: "0 10px",
    position: "relative",
    top: "0%",
    backgroundColor: "#e0e0e0",
    top: "0px",
    color: "white",
    fontSize: "30px",
    transition: "200ms",
    cursor: "pointer",
    "&:hover": {
      opacity: "1 !important",
    },
  },
  fullHeightHoverButton: {
    top: "calc(50% - 20px) !important",
  },
  playIconReversed: {
    transform: "rotate(180deg)",
  },

  buttonVisible: {
    opacity: "0.6",
  },
  buttonHidden: {
    opacity: "0",
  },
  next: {
    right: -45,
  },
  prev: {
    left: -45,
  },
  ff: {
    right: -82,
  },
  rew: {
    left: -82,
  },
};

const sanitizeProps = (props) => {
  const animation = props.animation !== undefined ? props.animation : "fade";
  const timeout =
    props.timeout !== undefined
      ? props.timeout
      : animation === "fade"
      ? 500
      : 200;

  return {
    children: props.children ? props.children : [],
    index: props.index !== undefined ? props.index : 0,
    strictIndexing:
      props.strictIndexing !== undefined ? props.strictIndexing : true,
    autoPlay: props.autoPlay !== undefined ? props.autoPlay : true,
    interval: props.interval !== undefined ? props.interval : 4000,
    indicators: props.indicators !== undefined ? props.indicators : true,
    navButtonsAlwaysInvisible:
      props.navButtonsAlwaysInvisible !== undefined
        ? props.navButtonsAlwaysInvisible
        : false,
    navButtonsAlwaysVisible:
      props.navButtonsAlwaysVisible !== undefined
        ? props.navButtonsAlwaysVisible
        : false,
    animation: animation,
    timeout: timeout,
    fullHeightHover:
      props.fullHeightHover !== undefined ? props.fullHeightHover : true,
    indicatorContainerProps: props.indicatorContainerProps,
    indicatorProps: props.indicatorProps,
    activeIndicatorProps: props.activeIndicatorProps,
    onChange: props.onChange !== undefined ? props.onChange : () => {},
    changeOnFirstRender:
      props.changeOnFirstRender !== undefined
        ? props.changeOnFirstRender
        : false,
    next: props.next !== undefined ? props.next : () => {},
    fastForward: props.next !== undefined ? props.fastforward : () => {},
    prev: props.prev !== undefined ? props.prev : () => {},
    fastRewind: props.next !== undefined ? props.fastRewind : () => {},

    className: props.className !== undefined ? props.className : "",
  };
};

class Carousel extends Component {
  constructor(props) {
    super(props);
    autoBind(this);

    this.state = {
      active: -1,
      prevActive: 0,
      displayed: 0,
    };

    this.timer = null;
  }

  componentDidMount() {
    const { index, changeOnFirstRender } = sanitizeProps(this.props);
    this.setActive(index, undefined, changeOnFirstRender);

    this.start();
  }

  componentWillUnmount() {
    this.stop();
  }

  componentDidUpdate(prevProps, prevState) {
    prevProps = sanitizeProps(prevProps);
    const { autoPlay, interval, children, index } = sanitizeProps(this.props);

    if (autoPlay !== prevProps.autoPlay || interval !== prevProps.interval) {
      this.reset();
    }

    if (children.length !== prevProps.children.length) {
      this.setActive(index);
    }

    if (prevProps.index !== index) {
      this.setActive(index);
    }
  }

  stop() {
    if (this.timer) {
      clearInterval(this.timer);
      this.timer = null;
    }
  }

  start() {
    const { autoPlay, interval } = sanitizeProps(this.props);

    if (autoPlay) {
      this.timer = setInterval(this.next, interval);
    }
  }

  reset() {
    const { autoPlay } = sanitizeProps(this.props);
    this.stop();

    if (autoPlay) {
      this.start();
    }
  }

  setActive(index, callback = () => {}, runCallbacks = true) {
    const { onChange, timeout, children, strictIndexing } = sanitizeProps(
      this.props
    );

    // if index is bigger than the children length, set it to be the last child (if strictIndexing)
    if (Array.isArray(children)) {
      if (strictIndexing && index > children.length - 1)
        index = children.length - 1;
      if (strictIndexing && index < 0) index = 0;
    } else {
      index = 0;
    }

    const prevActive = this.state.active;

    this.setState(
      {
        active: index,
        prevActive: prevActive,
        displayed: prevActive,
      },
      this.reset
    );

    setTimeout(() => {
      this.setState(
        {
          displayed: index,
        },
        () => {
          if (runCallbacks) {
            // Call user defined callbacks
            callback(index, prevActive);
            onChange(index, prevActive);
          }
        }
      );
    }, timeout);
  }

  next(event) {
    const { children, next } = sanitizeProps(this.props);

    const nextActive =
      this.state.active + 1 > children.length - 1 ? 0 : this.state.active + 1;

    this.setActive(nextActive, next);
    if (event) event.stopPropagation();
  }

  fastforward(event) {
    const { children, fastForward } = sanitizeProps(this.props);
    const nextActive =
      this.state.active + (children.length - this.state.active);

    this.setActive(nextActive, fastForward);

    if (event) event.stopPropagation();
  }
  prev(event) {
    const { children, prev } = sanitizeProps(this.props);

    const nextActive =
      this.state.active - 1 < 0 ? children.length - 1 : this.state.active - 1;

    this.setActive(nextActive, prev);

    if (event) event.stopPropagation();
  }
  fastrewind(event) {
    const { children, fastRewind } = sanitizeProps(this.props);
    const nextActive = 0;

    this.setActive(nextActive, fastRewind);

    if (event) event.stopPropagation();
  }
  render() {
    const {
      children,
      indicators,
      navButtonsAlwaysInvisible,
      navButtonsAlwaysVisible,
      animation,
      timeout,
      fullHeightHover,
      indicatorContainerProps,
      indicatorProps,
      activeIndicatorProps,
      className,
    } = sanitizeProps(this.props);

    const classes = this.props.classes;

    const buttonCssClassValue = `${classes.button} ${
      navButtonsAlwaysVisible ? classes.buttonVisible : classes.buttonHidden
    } ${fullHeightHover ? classes.fullHeightHoverButton : ""}`;
    const buttonWrapperCssClassValue = `${classes.buttonWrapper} ${
      fullHeightHover ? classes.fullHeightHoverWrapper : ""
    }`;

    const compareActiveDisplayed = () => {
      if (
        this.state.active === 0 &&
        this.state.prevActive === children.length - 1
      ) {
        return true;
      }

      if (
        this.state.active === children.length - 1 &&
        this.state.prevActive === 0
      ) {
        return false;
      }

      if (this.state.active > this.state.prevActive) {
        return true;
      }

      return false;
    };

    return (
      <div className={`${classes.container}`}>
        <div
          className={`${classes.root} ${className ? className : ""}`}
          onMouseEnter={this.stop}
          onMouseOut={this.reset}
        >
          {Array.isArray(children) ? (
            children.map((child, index) => {
              return (
                <CarouselItem
                  key={`carousel-item${index}`}
                  display={index === this.state.displayed ? true : false}
                  active={index === this.state.active ? true : false}
                  isNext={compareActiveDisplayed()}
                  child={child}
                  animation={animation}
                  timeout={timeout}
                  next={this.next}
                  prev={this.prev}
                />
              );
            })
          ) : (
            <CarouselItem
              key={`carousel-item0`}
              display={true}
              active={true}
              child={children}
              animation={animation}
              timeout={timeout}
              // next={this.next}
              // prev={this.prev}
            />
          )}

          {!navButtonsAlwaysInvisible && (
            <div className={`${buttonWrapperCssClassValue} ${classes.next}`}>
              <IconButton
                className={`${buttonCssClassValue} ${classes.next}`}
                onClick={this.next}
                aria-label="Next"
                style={{ borderRadius: "5px", padding: "0.5rem 1.2rem" }}
                disabled={
                  this.state.active === children.length - 1 ? true : false
                }
              >
                <PlayArrowIcon htmlColor="#7d7d7d" />{" "}
              </IconButton>
            </div>
          )}

          {!navButtonsAlwaysInvisible && (
            <div className={`${buttonWrapperCssClassValue} ${classes.prev} `}>
              <IconButton
                className={`${buttonCssClassValue}  ${classes.prev} ${classes.playIconReversed}`}
                onClick={this.prev}
                aria-label="Previous"
                style={{ borderRadius: "5px", padding: "0.5rem 1.2rem" }}
                disabled={this.state.active === 0 ? true : false}
              >
                <PlayArrowIcon htmlColor="#7d7d7d" />{" "}
              </IconButton>
            </div>
          )}
          {!navButtonsAlwaysInvisible && (
            <div className={`${buttonWrapperCssClassValue} ${classes.ff} `}>
              <IconButton
                className={`${buttonCssClassValue}  ${classes.ff}`}
                onClick={this.fastforward}
                aria-label="Fast-Forward"
                style={{ borderRadius: "5px", padding: "0.5rem 1.2rem" }}
                disabled={
                  this.state.active === children.length - 1 ? true : false
                }
              >
                <FastForwardIcon htmlColor="#757575" />{" "}
              </IconButton>
            </div>
          )}
          {!navButtonsAlwaysInvisible && (
            <div className={`${buttonWrapperCssClassValue} ${classes.rew} `}>
              <IconButton
                className={`${buttonCssClassValue}  ${classes.rew}`}
                onClick={this.fastrewind}
                aria-label="Fast-Rewind"
                style={{ borderRadius: "5px", padding: "0.5rem 1.2rem" }}
                disabled={this.state.active === 0 ? true : false}
              >
                <FastRewindIcon htmlColor="#757575" />{" "}
              </IconButton>
            </div>
          )}
          {indicators ? (
            <Indicators
              classes={classes}
              length={children.length}
              active={this.state.active}
              press={this.setActive}
              indicatorContainerProps={indicatorContainerProps}
              indicatorProps={indicatorProps}
              activeIndicatorProps={activeIndicatorProps}
            />
          ) : null}
        </div>
      </div>
    );
  }
}

function CarouselItem(props) {
  const swipeHandlers = useSwipeable({
    onSwipedLeft: () => props.next(),
    onSwipedRight: () => props.prev(),
  });

  return props.display ? (
    <div {...swipeHandlers} className="CarouselItem">
      {props.animation === "slide" ? (
        <Slide
          direction={
            props.active
              ? props.isNext
                ? "left"
                : "right"
              : props.isNext
              ? "right"
              : "left"
          }
          in={props.active}
          timeout={props.timeout}
        >
          <div>{props.child}</div>
        </Slide>
      ) : (
        <Fade in={props.active} timeout={props.timeout}>
          <div>{props.child}</div>
        </Fade>
      )}
    </div>
  ) : null;
}

function Indicators(props) {
  const classes = props.classes;

  let indicators = [];
  for (let i = 0; i < props.length; i++) {
    const style =
      props.indicatorProps !== undefined
        ? props.indicatorProps.style
        : undefined;
    let className =
      props.indicatorProps !== undefined
        ? props.indicatorProps.className
        : undefined;
    const activeStyle =
      props.activeIndicatorProps !== undefined
        ? props.activeIndicatorProps.style
        : undefined;
    const activeClassName =
      props.activeIndicatorProps !== undefined
        ? props.activeIndicatorProps.className
        : undefined;

    className =
      i === props.active
        ? `${classes.indicator} ${classes.active} ${activeClassName}`
        : `${classes.indicator} ${className}`;

    const item = (
      <FiberManualRecordIcon
        key={i}
        size="small"
        className={className}
        style={i === props.active ? activeStyle : style}
        onClick={() => {
          props.press(i);
        }}
      />
    );

    indicators.push(item);
  }

  const wrapperStyle =
    props.indicatorContainerProps !== undefined
      ? props.indicatorContainerProps.style
      : undefined;
  const wrapperClassName =
    props.indicatorContainerProps !== undefined
      ? props.indicatorContainerProps.className
      : "";

  return (
    <div
      className={`${classes.indicators} ${wrapperClassName}`}
      style={wrapperStyle}
    >
      {indicators}
    </div>
  );
}

// const CarouselContainer = styled.div`
//   display: flex;
//   justify-content: center;
//   align-items: center;
//   width: 70vw;
//   height: 300px;
//   background-color: #e0e0e0;
// `;
// const CustomButton = styled(Button)`
//   padding: 0.4rem 1.2rem;
//   position: absolute;
//   border-radius: 5px;
//   display: flex;
//   background-color: #e0e0e0;
//   border: 1px solid transparent;
// `;
// const Backward = styled(CustomButton)`
//   left: -11.5%;
//   top: -12%;
// `;
// const Forward = styled(CustomButton)`
//   right: -11.5%;
//   top: -12%;
// `;
// const FastRewind = styled(CustomButton)`
//   left: -5.8%;
//   top: -12%;
// `;
// const FastForward = styled(CustomButton)`
//   right: -5.8%;
//   top: -12%;
// `;
// const StyledPlayArrowIcon = styled(PlayArrowIcon)`
//   transform: rotate(180deg);
// `;
export default withStyles(styles)(Carousel);
